#include <iostream>
#include <array>
#include <algorithm>
#include <iomanip>

const uint32_t MAX_N = 2001;

typedef std::array<double, 2 * MAX_N> Vect; 

Vect cyc_conv(const Vect &a, const Vect &b, uint32_t n) {
	Vect ans;
	std::fill(ans.begin(), ans.end(), 0);
	for (uint32_t i = 0;i < n;i++) {
		for (uint32_t j = 0;j < n;j++) {
			ans[i + j] += a[i] * b[j];
		}
	}
	for (uint32_t i = 0;i < n;i++) {
		ans[i] += ans[i + n];
		ans[i + n] = 0;
	}
	return ans;
}

Vect op;
uint32_t n, k;
void input() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);

	std::cin >> n >> k;
	for (uint32_t i = 0;i < n;i++) {
		std::cin >> op[i];
	}
}

Vect solve() {
	Vect ans;
	std::fill(ans.begin(), ans.end(), 0);
	ans[0] = 1;

	Vect pow = op;

	for (uint32_t i = 1;i <= k;i *= 2) {
		if (k & i) {
			ans = cyc_conv(ans, pow, n);
		}
		pow = cyc_conv(pow, pow, n);
	}
	return ans;
}

void print(const Vect &v, std::ostream &s = std::cout) {
	for (uint32_t i = 0;i < n;i++) {
		s << std::fixed << std::setprecision(10) << v[i] << " ";
	}
	s << "\n";
}

int main() {
	input();
	print(solve());
}
