#include <iostream>
#include <array>
#include <algorithm>
#include <iomanip>
#include <complex>
#include <cmath>

const double PI = 3.14159265359;


const uint32_t BIT_CNT = 15;
const uint32_t MAX_N = 1ULL << 14;
const uint32_t FFT_N = 2 * MAX_N;

typedef std::complex<double> Comp;
typedef std::array<Comp, 2 * MAX_N> Vect; 

uint32_t bit_reverse(uint32_t a) {
	uint32_t ans = 0;
	for (uint32_t i = 0;i < BIT_CNT;i++) {
		ans = (ans << 1) | (a & 1);
		a >>= 1;
	}
	return ans;
}

Vect fft(const Vect &a, bool inverse) {
	Vect v = a;
	for (uint32_t i = 0;i < FFT_N;i++) {
		uint32_t j = bit_reverse(i);
		if (j < i) {
			std::swap(v[i], v[j]);
		}
	}
	for (uint32_t l = 1;l <= BIT_CNT;l++) {
		uint32_t len = 1ULL << l;
		uint32_t hLen = len / 2;
		Comp root = {cos(PI / hLen), sin(PI / hLen)};
		if (inverse) {
			root = 1.0 / root;
		}
		for (uint32_t j = 0;j < FFT_N;j += len) {
			Comp prod = 1;
			for (uint32_t k = j;k < j + hLen;k++) {
				Comp a = v[k];
				Comp b = v[k + hLen];

				v[k] = a + b * prod;
				v[k + hLen] = a - b * prod;
				prod = prod * root;
			}
		}
	}
	if (inverse) {
		for (uint32_t i = 0;i < FFT_N;i++) {
			v[i] /= FFT_N;
		}
	}
	return v;
}

Vect mult(const Vect &a, const Vect &b) {
	Vect ans;
	for (uint32_t i = 0;i < FFT_N;i++) {
		ans[i] = a[i] * b[i];
	}
	return ans;
}

Vect cyc_conv(const Vect &a, const Vect &b, uint32_t n) {
	Vect afft = fft(a, false);
	Vect bfft = fft(b, false);
	Vect ansfft = mult(afft, bfft);
	Vect ans = fft(ansfft, true);
	//std::fill(ans.begin(), ans.end(), 0);
	//for (uint32_t i = 0;i < n;i++) {
	//	for (uint32_t j = 0;j < n;j++) {
	//		ans[i + j] += a[i] * b[j];
	//	}
	//}
	for (uint32_t i = 0;i < n;i++) {
		ans[i] += ans[i + n];
		ans[i + n] = 0;
	}
	return ans;
}

Vect op;
uint32_t n, k;
void input() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);

	std::cin >> n >> k;
	for (uint32_t i = 0;i < n;i++) {
		std::cin >> op[i];
	}
}

Vect solve() {
	Vect ans;
	std::fill(ans.begin(), ans.end(), 0);
	ans[0] = 1;

	Vect pow = op;

	for (uint32_t i = 1;i <= k;i *= 2) {
		if (k & i) {
			ans = cyc_conv(ans, pow, n);
		}
		pow = cyc_conv(pow, pow, n);
	}
	return ans;
}

void print(const Vect &v, std::ostream &s = std::cout) {
	for (uint32_t i = 0;i < n;i++) {
		s << std::fixed << std::setprecision(10) << v[i].real() << " ";
	}
	s << "\n";
}

int main() {
	input();
	print(solve());
}
