#include <bits/stdc++.h>

using namespace std;

struct edge{
    long long to, len;

    edge(){}
    edge(long long to, long long len){
        this->to = to;
        this->len = len;
    }
};

struct cmp{
    bool operator()(pair<int, long long> lvalue, pair<int, long long> rvalue){
        return lvalue.second > rvalue.second;
    }
};

const long long N = 5e4 + 7;
const long long inf = 1e18;

vector<edge> adjx[N], adjy[N];
bool usedx[N], usedy[N];
long long distx[N], disty[N];
long long nx, ny, mx, my;

long long dijkstrax(int from, int to){
    priority_queue<pair<int, long long>, vector<pair<int, long long> >, cmp> pq;

    for(int i = 1; i <= nx; i++){
        distx[i] = inf;
    }

    pq.push({from, 0});
    distx[from] = 0;

    while(!pq.empty()){
        pair<int, long long> p = pq.top();
        pq.pop();

        int u = p.first;
        long long d = p.second;

        if(usedx[u]){
            continue;
        }
        usedx[u] = true;

        if(u == to){
            return d;
        }

        for(edge e: adjx[u]){
            if(e.len + d < distx[e.to]){
                distx[e.to] = e.len + d;
                pq.push({e.to, distx[e.to]});
            }
        }
    }

    return -1;
}

long long dijkstray(int from, int to){
    priority_queue<pair<int, long long>, vector<pair<int, long long> >, cmp> pq;

     for(int i = 1; i <= ny; i++){
        disty[i] = inf;
    }

    pq.push({from, 0});
    disty[from] = 0;

    while(!pq.empty()){
        pair<int, long long> p = pq.top();
        pq.pop();

        int u = p.first;
        long long d = p.second;

        if(usedy[u]){
            continue;
        }
        usedy[u] = true;

        if(u == to){
            return d;
        }

        for(edge e: adjy[u]){
            if(e.len + d < disty[e.to]){
                disty[e.to] = e.len + d;
                pq.push({e.to, disty[e.to]});
            }
        }
    }

    return -1;
}

struct mst{
    int from, to, len;
    bool dim;

    mst(){}
    mst(int from, int to, int len, bool dim){
        this->from = from;
        this->to = to;
        this->len = len;
        this->dim = dim;
    }

    friend bool operator<(mst lvalue, mst rvalue){
        return lvalue.len < rvalue.len;
    }
};

vector<mst> ed;
int parx[N], pary[N], szx[N], szy[N];

int find_parx(int u){
    if(u == parx[u]){
        return u;
    }

    parx[u] = find_parx(parx[u]);

    return parx[u];
}

bool unitex(int u, int v){
    if(find_parx(u) == find_parx(v)){
        return false;
    }

    if(szx[parx[u]] < szx[parx[v]]){
        swap(u, v);
    }

    szx[parx[u]] += szx[parx[v]];
    parx[parx[v]] = parx[u];

    return true;
}

int find_pary(int u){
    if(u == pary[u]){
        return u;
    }

    pary[u] = find_pary(pary[u]);

    return pary[u];
}

bool unitey(int u, int v){
    if(find_pary(u) == find_pary(v)){
        return false;
    }

    if(szy[pary[u]] < szy[pary[v]]){
        swap(u, v);
    }

    szy[pary[u]] += szy[pary[v]];
    pary[pary[v]] = pary[u];

    return true;
}

int main(){
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    long long task;

    cin >> task;

    cin >> nx >> mx;

    for(long long i = 0; i < mx; i++){
        long long from, to, len;

        cin >> from >> to >> len;

        adjx[from].push_back(edge(to, len));
        adjx[to].push_back(edge(from, len));

        ed.push_back(mst(from, to, len, true));
    }
    cin >> ny >> my;

    for(long long i = 0; i < my; i++){
        long long from, to, len;

        cin >> from >> to >> len;

        adjy[from].push_back(edge(to, len));
        adjy[to].push_back(edge(from, len));

        ed.push_back(mst(from, to, len, false));
    }

    if(task == 1){
        cout << dijkstrax(1, nx) + dijkstray(1, ny) << endl;

        return 0;
    }

    sort(ed.begin(), ed.end());

    int cntx = 0, cnty = 0;
    long long ans = 0;

    for(int i = 1; i <= nx; i++){
        parx[i] = i;
        szx[i] = 1;
    }

    for(int i = 1; i <= ny; i++){
        pary[i] = i;
        szy[i] = 1;
    }

    for(mst t: ed){
        if(t.dim){
            if(!unitex(t.from, t.to)){
                continue;
            }
            ans += (long long)(ny - cnty) * t.len;
            cntx++;
        }
        else{
            if(!unitey(t.from, t.to)){
                continue;
            }
            ans += (long long)(nx - cntx) * t.len;
            cnty++;
        }
    }

    cout << ans << "\n";

    return 0;
}
/*
1
3 2
2 1 15
3 1 14
3 2
2 1 15
3 2 15
*/
/*
2
3 2
2 1 15
3 1 14
3 2
2 1 15
3 2 15
*/
