#include <iostream>
#include <vector>
#include <queue>
#include <utility>
#include <algorithm>
using namespace std;

const int MAXN = 50005;

int nx, mx, ny, my;
vector<pair<int, int>> gx[MAXN], gy[MAXN];

int subtask;

long long distx[MAXN][2], disty[MAXN][2];

vector<pair<int, pair<int, int>>> edgesx, edgesy;

void Init()
{
    ios :: sync_with_stdio(false);
    cin.tie(NULL); cout.tie(NULL);

    cin >> subtask;

    cin >> nx >> mx;
    for (int i = 1; i <= mx; ++ i)
    {
        int u, v, w;
        cin >> u >> v >> w;
        gx[u].emplace_back(v, w);
        gx[v].emplace_back(u, w);
        edgesx.emplace_back(w, make_pair(u, v));
    }

    cin >> ny >> my;
    for (int i = 1; i <= my; ++ i)
    {
        int u, v, w;
        cin >> u >> v >> w;
        gy[u].emplace_back(v, w);
        gy[v].emplace_back(u, w);
        edgesy.emplace_back(w, make_pair(u, v));
    }

    for (int i = 1; i <= nx; ++ i)
    {
        distx[i][0] = 1ll << 60;
    }
    for (int i = 1; i <= ny; ++ i)
    {
        disty[i][0] = 1ll << 60;
    }
}

long long DijkstraX()
{
    priority_queue<pair<long long, int>> pq;
    distx[1][0] = 0;
    pq.emplace(0, 1);

    while (!pq.empty())
    {
        long long d = -pq.top().first;
        int u = pq.top().second;

        pq.pop();

        if (distx[u][1]) continue;
        distx[u][1] = 1;

        for (auto e : gx[u])
        {
            if (d + e.second < distx[e.first][0])
            {
                distx[e.first][0] = d + e.second;
                pq.emplace(-distx[e.first][0], e.first);
            }
        }
    }

    return distx[nx][0];
}

long long DijkstraY()
{
    priority_queue<pair<long long, int>> pq;
    disty[1][0] = 0;
    pq.emplace(0, 1);

    while (!pq.empty())
    {
        long long d = -pq.top().first;
        int u = pq.top().second;

        pq.pop();

        if (disty[u][1]) continue;
        disty[u][1] = 1;

        for (auto e : gy[u])
        {
            if (d + e.second < disty[e.first][0])
            {
                disty[e.first][0] = d + e.second;
                pq.emplace(-disty[e.first][0], e.first);
            }
        }
    }

    return disty[ny][0];
}

vector<pair<int, pair<int, int>>> edges;

int dsu[MAXN];

int root(int x)
{
    if (dsu[x] == x) return x;
    else return dsu[x] = root(dsu[x]);
}

long long KruskalXY()
{
    for (int i = 0; i < MAXN; ++ i)
    {
        dsu[i] = i;
    }

    for (int i = 0; i < edgesx.size(); ++ i)
    {
        for (int j = 1; j <= ny; ++ j)
        {
            //cout << edgesx[i].first << ' ' << (j - 1) * ny + edgesx[i].second.first << ' ' << (j - 1) * ny + edgesx[i].second.second << endl;
            edges.emplace_back(edgesx[i].first, make_pair((j - 1) * ny + edgesx[i].second.first, (j - 1) * ny + edgesx[i].second.second));
        }
    }

    for (int i = 0; i < edgesy.size(); ++ i)
    {
        for (int j = 1; j <= nx; ++ j)
        {
            //cout << edgesy[i].first << ' ' << (edgesy[i].second.first - 1) * ny + j << ' ' << (edgesy[i].second.second - 1) * ny + j << endl;
            edges.emplace_back(edgesy[i].first, make_pair((edgesy[i].second.first - 1) * ny + j, (edgesy[i].second.second - 1) * ny + j));
        }
    }

    sort(edges.begin(), edges.end());

    //cout <<  "----" << endl;

    long long ans = 0;
    for (int i = 0; i < edges.size(); ++ i)
    {
        //cout << edges[i].second.first << ' ' << edges[i].second.second << ' ' << edges[i].first << endl;

        int rx = root(edges[i].second.first);
        int ry = root(edges[i].second.second);

        if (rx != ry)
        {
            //cout << "ok " << endl;
            dsu[rx] = ry;
            ans += edges[i].first;
        }
    }

    return ans;
}

int main()
{
    Init();

    if (subtask == 1)
    {
        cout << DijkstraX() + DijkstraY() << endl;
        return 0;
    }
    if (subtask == 2)
    {
        cout << KruskalXY() << endl;
        return 0;
    }
}

/*
1
3 2
2 1 15
3 1 14
3 2
2 1 15
3 2 15

-----------


2
3 2
2 1 15
3 1 14
3 2
2 1 15
3 2 15

*/
