#include <bits/stdc++.h>

const int32_t MAX_N = 5e4;

struct State {
	int32_t first;
	int64_t second;

	State() {}
	State(int32_t _first, int64_t _second) : first(_first), second(_second) {}

	bool operator< (const State &o) const {
		return second > o.second;
	}
};

struct Edge {
	int32_t first, second, w;

	Edge() {}
	Edge(int32_t _first, int32_t _second, int32_t _w) : first(_first), second(_second), w(_w) {}

	bool operator< (const Edge &o) const {
		return w < o.w;
	}
};

struct DSU {
	int32_t par[MAX_N + 5], sizes[MAX_N + 5];

	void init(int32_t _n) {
		for(int32_t i = 1; i <= _n; i++) {
			par[i] = i;
			sizes[i] = 1;
		}
	}

	int32_t find_par(int32_t x) {
		if(par[x] == x) {
			return x;
		}

		par[x] = find_par(par[x]);
		return par[x];
	}

	void merge_sets(int32_t u, int32_t v) {
		if(sizes[u] < sizes[v]) {
			par[u] = v;
			sizes[v] += sizes[u];
		}
		else if(sizes[u] >= sizes[v]) {
			par[v] = u;
			sizes[u] += sizes[v];
		}
	}
};

bool vis[2][MAX_N + 5];
int64_t dists[2][MAX_N + 5];
std::vector< std::pair< int32_t, int32_t > > g[2][MAX_N + 5];
DSU dsu[2];

void dijkstra(int32_t x, int32_t start) {
	std::priority_queue< State > pq;

	dists[x][start] = 0;
	pq.push(State(start, 0));

	while(!pq.empty()) {
		auto curr = pq.top();
		pq.pop();

		int32_t p = curr.first;
		int64_t dist = curr.second;

		if(vis[x][p]) {
			continue;
		}
		vis[x][p] = true;

		for(auto &q : g[x][p]) {
			int32_t nxt = q.first;
			int64_t newD = dist + q.second;

			if(!vis[x][nxt] && (dists[x][nxt] == -1 || dists[x][nxt] > newD)) {
				pq.push(State(nxt, newD));
				dists[x][nxt] = newD;
			}
		}
	}
}

int64_t calc_mst(int32_t x) {
	std::vector< Edge > edges;
	
	for(int32_t i = 1; i <= MAX_N; i++) {
		for(auto &p : g[x][i]) {
			edges.push_back(Edge(i, p.first, p.second));
		}
	}

	std::sort(edges.begin(), edges.end());

	int64_t ans = 0;
	for(auto &e : edges) {
		int32_t parU = dsu[x].find_par(e.first);
		int32_t parV = dsu[x].find_par(e.second);

		if(parU != parV) {
			ans += e.w;
			dsu[x].merge_sets(parU, parV);
		}
	}

	return ans;
}

int main() {
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(nullptr);

	int32_t task;
	std::cin >> task;

	int32_t nx, mx;
	std::cin >> nx >> mx;

	for(int32_t i = 0; i < mx; i++) {
		int32_t u, v, w;
		std::cin >> u >> v >> w;

		g[0][u].push_back({ v, w });
		g[0][v].push_back({ u, w });
	}

	int32_t ny, my;
	std::cin >> ny >> my;

	for(int32_t i = 0; i < my; i++) {
		int32_t u, v, w;
		std::cin >> u >> v >> w;

		g[1][u].push_back({ v, w });
		g[1][v].push_back({ u, w });
	}

	if(task == 1) {
		memset(dists, -1, sizeof(dists));

		dijkstra(0, 1);
		dijkstra(1, 1);

		std::cout << dists[0][nx] + dists[1][ny] << '\n';
	}
	else {
		dsu[0].init(nx);
		dsu[1].init(ny);

		int64_t p = calc_mst(0);
		int64_t q = calc_mst(1);
	
		std::cout << std::min(nx * q + p, ny * p + q) << '\n';
	}
}
